﻿namespace Battleship.Entities;

/// <summary>
/// Класс-сущность Линкор
/// </summary>
public class EntityBattleship : EntityWarship
{
    /// /// <summary>
    /// Признак (опция) отсека для ракет
    /// </summary>
    public bool Compartment { get; private set; }
    /// <summary>
    /// Признак (опция) наличия башни
    /// </summary>
    public bool Tower { get; private set; }
    /// <summary>
    /// Признак (опция) наличия палубы
    /// </summary>
    public bool BodyDeck { get; private set; }
    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }
    public void SetAdditionalColor(Color color)
    {
        AdditionalColor = color;
    }

    /// <summary>
    /// Перемещение линкора
    /// </summary>
    public double Step => Speed * 100 / Weight;

    /// <summary>
    /// Инициализация полей объекта-класса линкора
    /// </summary>
    /// <param name="speed"></param>
    /// <param name="weight"></param>
    /// <param name="bodyColor"></param>
    public EntityBattleship(int speed, double weight, Color bodyColor, bool compartment, bool tower, bool bodyDeck, Color additionalColor) : base(speed, weight, bodyColor)
    {
        Compartment = compartment;
        Tower = tower;
        BodyDeck = bodyDeck;
        AdditionalColor = additionalColor;
    }

    /// <summary>
    /// Получение строки с значением свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityBattleship), Speed.ToString(), Weight.ToString(), BodyColor.Name,  Compartment.ToString(), Tower.ToString(), BodyDeck.ToString(), AdditionalColor.Name};
    }

    /// <summary>
    /// Создание продвинутого объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityBattleship? CreateEntityBattleship(string[] strs)
    {
        if (strs.Length != 8 || strs[0] != nameof(EntityBattleship))
        {
            return null;
        }
        return new EntityBattleship(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]), Convert.ToBoolean(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]), Color.FromName(strs[7]));
    }
}