﻿namespace Battleship.Entities;
/// <summary>
/// Класс-сущность Военный корабль
/// </summary>
public class EntityWarship
{
    /// <summary>
    /// Скорость
    /// </summary>
    public int Speed { get; private set; }
    /// <summary>
    /// Вес
    /// </summary>
    public double Weight { get; private set; }
    /// <summary>
    /// Основной цвет 
    /// </summary>
    public Color BodyColor { get; private set; }

    /// <summary>
    /// Основной цвет
    /// </summary>
    /// <param name="color"></param>
    public void SetBodyColor(Color color)
    {
        BodyColor = color;
    }

    /// <summary>
    /// Перемещение военного корабля
    /// </summary>
    public double Step => Speed * 100 / Weight;

    /// <summary>
    /// Конструктор сущности
    /// </summary>
    /// <param name="speed"></param>
    /// <param name="weight"></param>
    /// <param name="bodyColor"></param>
    public EntityWarship(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }

    /// <summary>
    /// Получение строки с значением свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityWarship), Speed.ToString(), Weight.ToString(), BodyColor.Name };
    }
     
    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityWarship? CreateEntityWarship(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityWarship))
        {
            return null;
        }

        return new EntityWarship(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }
}
